(*
 * Copyright 2005, 2006, 2007 Aaron R. Bradley and Zohar Manna
 *
 * This file is part of PiVC.
 * 
 * PiVC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 * 
 * PiVC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

open Genlex

let cvc_timeout = ref 3600

let cvc_exec = ref ""

let sat asrt =
  let fn = Filename.temp_file "piVC_" ".smt" in
  let oc = open_out fn in
  output_string oc asrt;
  close_out oc;

  let read_output ic =
    let parse_output stream =
      let input = ref false in
      let rv = ref true in
      let parse_next () =
	match Stream.next stream with
	| Kwd "Valid" -> input := true; rv := false
	| Kwd "InValid" -> input := true; rv := true
	| _ -> ()
      in
      try
	while true do
	  parse_next ()
	done;
	!rv
      with
      | Stream.Failure -> 
	  if not !input then print_endline "INTERRUPTED (CVC Lite)";
	  !rv
    in
    let lexer = 
      make_lexer ["Valid"; "InValid"; "."; ","; "("; ")"] 
    in
    parse_output (lexer (Stream.of_channel ic))
  in

  let cmd = 
    !cvc_exec ^ " -timeout " 
    ^ (string_of_int !cvc_timeout) 
    ^ " -lang smtlib < " ^ fn
  in
  print_endline "Calling CVC Lite...";

  let ic, oc = Unix.open_process cmd in
  let rv = read_output ic in
  Unix.close_process (ic, oc);
  Unix.unlink fn;
  rv
