(*
 * Copyright 2005, 2006, 2007 Aaron R. Bradley and Zohar Manna
 *
 * This file is part of PiVC.
 * 
 * PiVC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 * 
 * PiVC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

(* Failure to translate *)
exception Fail


type sent =
  | True
  | False
  | Pred of pred
  | EQ of value * value
  | EQZ of nval
  | GEZ of nval
  | GTZ of nval
  | Not of sent
  | And of sent * sent
  | Or of sent * sent
  | Exists of Type.typ * string * sent

and read =
  | Var of string
  | ArrayAcc of array * nval
  | StructAcc of read * string
  | FnCall of string * value list

and pred =
  | PVar of read
  | PPred of string * value list

and array = (nval * value) list * read

and nval =
  | Constant of int
  | Term of int * read
  | Sum of nval list

and value =
  | Bool of sent
  | Read of read
  | Array of array
  | Number of nval


val value_of_expr : Interface.prgm -> Interface.fn -> Vcg.expr -> value


val string_of_sent : sent -> string

val string_of_read : read -> string

val string_of_pred : pred -> string

val string_of_array : array -> string

val string_of_nval : nval -> string

val string_of_value : value -> string


val annoi_string : value -> string

val smt_string : value -> string
