import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

public class PiDebugger extends AbstractTableModel {

    private class Step {
	public Step() {
	    function = "";
	    line = -1;
	    vv = new ArrayList();
	}
	public String function;
	public int line;
	public ArrayList vv;
    }

    private class VV {
	public String var;
	public String val;
    }

    Pi pi;

    ArrayList record;
    int stepi;

    public PiDebugger() {
	record = new ArrayList();
	stepi = -1;
    }

    public void setPi(Pi p) {
	pi = p;
    }

    public void reset() {
	record.clear();
	stepi = -1;
    }

    public void newStep(String fn, int ln) {
	Step step = new Step();
	step.function = fn;
	step.line = ln;
	record.add(step);
	if (stepi == -1) next();
    }

    public void setValue(String var, String val) {
	assert record.size() > 0;
	Step step = (Step) record.get(record.size() - 1);
	VV vv = new VV();
	vv.var = var;
	vv.val = val;
	step.vv.add(vv);
    }

    public void setRV(String rv) {
	pi.firePiReturnValue(rv);
    }

    public void setError(String err) {
	pi.firePiError(err);
    }

    private void line() {
	assert stepi >= 0 && stepi < record.size();
	Step step = (Step) record.get(stepi);
	Integer[] lines = new Integer[1];
	lines[0] = new Integer(step.line);
	pi.firePiLines(lines);
    }

    public void first() {
	if (record.size() > 0) {
	    stepi = 0;
	    fireTableDataChanged();
	    line();
	}
    }

    public void last() {
	if (record.size() > 0) {
	    stepi = record.size() - 1;
	    fireTableDataChanged();
	    line();
	}
    }

    public void next() {
	if (stepi < record.size() - 1) {
	    stepi++;
	    fireTableDataChanged();
	    line();
	}
    }

    public void prev() {
	if (stepi > 0) {
	    stepi--;
	    fireTableDataChanged();
	    line();
	}
    }

    public String getColumnName(int col) {
	assert col >= 0 && col < 2;
	if (col == 0) return "Variable";
	if (col == 1) return "Value";
	return "";
    }

    public int getRowCount() {
	if (stepi < 0 || stepi >= record.size()) return 0;
	Step step = (Step) record.get(stepi);
	return step.vv.size();
    }

    public int getColumnCount() {
	return 2;
    }

    public Object getValueAt(int row, int col) {
	assert stepi >= 0 && stepi < record.size();
	Step step = (Step) record.get(stepi);
	assert row >= 0 && row < step.vv.size();
	VV vv = (VV) step.vv.get(row);
	assert col >= 0 && col < 2;
	if (col == 0) return vv.var;
	if (col == 1) return vv.val;
	return "";
    }

}
