import java.io.*;
import java.util.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.plaf.*;
import javax.swing.text.*;

public class PiGUI implements PiListener {

    static String email;
    static String libdir = "";
    static String pvc = "./pvc";
    static String cvcl = "";
    static String yices = "";
    static String cvctimeout = "600";
    static int fontsz = 14;

    JPanel mainPanel, codePanel, infoPanel;
    JMenuBar menuBar;

    JTextPane codePane;
    JTextArea outputArea;

    DefaultHighlighter.DefaultHighlightPainter hlp = 
        new DefaultHighlighter.DefaultHighlightPainter(Color.YELLOW);

    JComboBox functionCB, vcCB;
    JTextArea vcArea;

    PiDebugger debugger;
    JTextArea evalArea, debugArea;

    JFileChooser fileChooser;

    Pi pi;
    boolean compiling = false;

    boolean proveVCs = true;
    boolean runtimeAssertions = false;
    boolean ig = false;

    WindowListener exitListener;

    public PiGUI(int size) {
        debugger = new PiDebugger();
        pi = new Pi(pvc, cvcl.equals("") ? yices : cvcl, cvctimeout);
        pi.setDebugger(debugger);
        pi.addListener(this);

        codePanel = new JPanel();
        codePanel.setLayout(new GridLayout(1, 1));
        codePanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        infoPanel = new JPanel();
        infoPanel.setLayout(new GridLayout(1, 1));
        infoPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        addWidgets(size);

        mainPanel = new JPanel();
        mainPanel.setLayout(new GridLayout(1, 2));
        mainPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

	JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				       codePanel,
				       infoPanel);
	sp.setOneTouchExpandable(true);
	//sp.setDividerLocation(.5);

        mainPanel.add(sp);

        exitListener = new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    exit();
                }
            };
    }

    private void error(Exception e) {
        System.out.println("<error>");
        e.printStackTrace(System.out);
    }

    static void enableEmail(String addr) {
        email = addr;
    }

    static void set_pvc(String pvc) {
	PiGUI.pvc = pvc;
    }

    static void set_cvcl(String cvcl) {
	PiGUI.cvcl = cvcl;
    }

    static void set_yices(String yices) {
	PiGUI.yices = yices;
    }

    static void set_cvctimeout(String cvctimeout) {
	PiGUI.cvctimeout = cvctimeout;
    }

    static void setLibDir(String libdir) {
	PiGUI.libdir = libdir;
    }

    static void setFontSize(int sz) {
	fontsz = sz;
	setFontSize();
    }

    private static void setFontSize() {
        try {
            FontUIResource font = 
                new FontUIResource("SansSerif", Font.PLAIN, fontsz);
            FontUIResource cfont = 
                new FontUIResource("Courier", Font.PLAIN, fontsz);
            UIManager.put("TextArea.font", cfont);
            UIManager.put("TextPane.font", cfont);
            UIManager.put("Button.font", font);
            UIManager.put("CheckBox.font", font);
            UIManager.put("ComboBox.font", font);
            UIManager.put("Menu.font", font);
            UIManager.put("CheckBoxMenuItem.font", font);
            UIManager.put("MenuItem.font", font);
            UIManager.put("Panel.font", font);
            UIManager.put("TitledBorder.font", font);
            UIManager.put("Table.font", cfont);
            UIManager.put("Frame.font", font);
        }
        catch(Exception e) {
            e.printStackTrace(System.out);
        }
    }

    private void addWidgets(int size) {
        int width = 2 * size / 3;

        /* --- File chooser --- */

        fileChooser = new JFileChooser();
        fileChooser.addChoosableFileFilter(new FileFilter() {
                public boolean accept(File file) {
                    if (file.isDirectory()) return true;
                    String name = file.getName();
                    if (name.length() < 3) return true;
                    String extension = name.substring(name.length() - 2);
                    return 
                        (extension != null
                         && (extension.equals("pi")
                             || extension.equals("PI")));
                }
                public String getDescription() {
                    return "pi programs";
                }
            });

        /* --- Menu bar --- */

        menuBar = new JMenuBar();
        JMenu menu;
        JMenuItem item;

        menu = new JMenu("File");
        menuBar.add(menu);

        item = new JMenuItem("Open", KeyEvent.VK_O);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, 
                                                   ActionEvent.CTRL_MASK));
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (fileChooser.showOpenDialog(mainPanel) 
                        == JFileChooser.APPROVE_OPTION) {
                        open(fileChooser.getSelectedFile());
                    }
                }
            });

        item = new JMenuItem("Save", KeyEvent.VK_S);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, 
                                                   ActionEvent.CTRL_MASK));
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (fileChooser.showSaveDialog(mainPanel) 
                        == JFileChooser.APPROVE_OPTION) {
                        save(fileChooser.getSelectedFile());
                    }                    
                }
            });

        menu.addSeparator();

        item = new JMenuItem("Quit", KeyEvent.VK_Q);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, 
                                                   ActionEvent.CTRL_MASK));
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    exit();
                }
            });

        menu = new JMenu("Compile");
        menu.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        menuBar.add(menu);

        item = new JCheckBoxMenuItem("Only Generate VCs");
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    proveVCs = !proveVCs;
                    pi.setProve(proveVCs);
                }
            });

        item = new JCheckBoxMenuItem("Runtime Assertions");
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    runtimeAssertions = !runtimeAssertions;
                    pi.setRuntimeAssertions(runtimeAssertions);
                }
            });

	/*
        item = new JCheckBoxMenuItem("Generate Invariants");
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    ig = !ig;
                    pi.setGenerateInvariants(ig);
                }
            });
	*/

        menu.addSeparator();

        item = new JMenuItem("Compile", KeyEvent.VK_C);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, 
                                                   ActionEvent.ALT_MASK));
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (compiling) pi.stop();

                    outputArea.setText("");
                    functionCB.removeAllItems();
                    vcCB.removeAllItems();
                    vcArea.setText("");

		    compiling = 
			pi.compile(codePane.getText(), 
				   libdir);
                }
            });

        item = new JMenuItem("Abort", KeyEvent.VK_A);
        item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, 
                                                   ActionEvent.ALT_MASK));
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (compiling) pi.stop();
                }
            });

        /*
        menu = new JMenu("Font");
        menuBar.add(menu);

        item = new JMenuItem("Smaller", KeyEvent.VK_S);
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    fontSize -= 2;
                    setFontSize();
                }
            });

        item = new JMenuItem("Larger", KeyEvent.VK_L);
        menu.add(item);
        item.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    fontSize += 2;
                    setFontSize();
                }
            });
        */

        /* --- Code pane --- */

        codePane = new JTextPane();
        codePane.addFocusListener(new FocusListener() {
                public void focusGained(FocusEvent e) {
                    codePane.getHighlighter().removeAllHighlights();
                }
                public void focusLost(FocusEvent e) {
                }
            });

        codePane.setCaretPosition(0);
        codePane.setMargin(new Insets(5, 5, 5, 5));
        JScrollPane scrollPane = new JScrollPane(codePane);
        scrollPane.setPreferredSize(new Dimension(width, size));

        codePanel.add(scrollPane);

        /* --- Info pane --- */

        JTabbedPane tabbedPane = new JTabbedPane();

        outputArea = new JTextArea();
        outputArea.setEditable(false);
        scrollPane = new JScrollPane(outputArea);
        scrollPane.setPreferredSize(new Dimension(width, size));

        tabbedPane.addTab("Raw Output", scrollPane);

        JPanel vcPanel = new JPanel();
        vcPanel.setLayout(new BorderLayout());
        vcPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        Box box = Box.createVerticalBox();
        vcPanel.add(box, BorderLayout.NORTH);

        functionCB = new JComboBox();
        functionCB.addActionListener(new ActionListener() {
                Object old;
                public void actionPerformed(ActionEvent e) {
                    String fname = (String) functionCB.getSelectedItem();

                    if (old != null && old.equals(fname)) return;
                    old = fname;

                    vcCBLoad(fname);
                }
            });
        box.add(functionCB);

        vcCB = new JComboBox();
        vcCB.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String vcn = (String) vcCB.getSelectedItem();
                    if (vcn != null) {
			String vcname = vcName(vcn);
                        String lines = vcLines(vcn);
                        highlight(getLines(lines));
                        vcArea.setText(pi.getVC(vcname));
                    }
                }
            });
        box.add(vcCB);

        vcArea = new JTextArea();
        vcArea.setEditable(false);
        scrollPane = new JScrollPane(vcArea);
        vcPanel.add(scrollPane, BorderLayout.CENTER);

        tabbedPane.addTab("Verification Conditions", vcPanel);

        JPanel debugPanel = new JPanel();
        debugPanel.setLayout(new BorderLayout());
        debugPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        box = Box.createVerticalBox();
        debugPanel.add(box, BorderLayout.NORTH);

        evalArea = new JTextArea();
        evalArea.setEditable(true);
        evalArea.setPreferredSize(new Dimension(width, 50));
        scrollPane = new JScrollPane(evalArea);
        box.add(scrollPane);

        JButton debugEval = new JButton("Evaluate");
        debugEval.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (compiling) pi.stop();

                    outputArea.setText("");

                    String expr = evalArea.getText();
                    if (expr != null && !"".equals(expr))
			compiling = 
			    pi.eval(codePane.getText(), 
				    expr,
				    libdir);
                }
            });
        Box hbox = Box.createHorizontalBox();
        hbox.add(Box.createHorizontalGlue());
        hbox.add(debugEval);
        hbox.add(Box.createHorizontalGlue());
        box.add(hbox);

        debugArea = new JTextArea();
        debugArea.setEditable(false);
        debugArea.setPreferredSize(new Dimension(width, 50));
        scrollPane = new JScrollPane(debugArea);
        box.add(scrollPane);

        box.add(Box.createVerticalStrut(10));

        JTable dt = new JTable(debugger);
        dt.setShowGrid(true);
        scrollPane = new JScrollPane(dt);
        debugPanel.add(scrollPane, BorderLayout.CENTER);

        box = Box.createVerticalBox();
        box.add(Box.createVerticalStrut(10));
        hbox = Box.createHorizontalBox();
        box.add(hbox);
        debugPanel.add(box, BorderLayout.SOUTH);

        hbox.add(Box.createHorizontalGlue());

        JButton debugFirst = new JButton("<<");
        debugFirst.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    debugger.first();
                }
            });
        hbox.add(debugFirst);

        JButton debugPrev = new JButton("<");
        debugPrev.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    debugger.prev();
                }
            });
        hbox.add(debugPrev);

        JButton debugNext = new JButton(">");
        debugNext.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    debugger.next();
                }
            });
        hbox.add(debugNext);

        JButton debugLast = new JButton(">>");
        debugLast.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    debugger.last();
                }
            });
        hbox.add(debugLast);

        hbox.add(Box.createHorizontalGlue());

        tabbedPane.addTab("Debugger", debugPanel);

        infoPanel.add(tabbedPane);

        /* --- Borders --- */

        codePanel.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("Code"),
            BorderFactory.createEmptyBorder(5, 5, 5, 5)));
        infoPanel.setBorder(BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder("Output"),
            BorderFactory.createEmptyBorder(5, 5, 5, 5)));
    }

    private void open(File file) {
        codePane.getHighlighter().removeAllHighlights();
        try {
            BufferedReader in = new BufferedReader(new FileReader(file));
            codePane.read(in, null);

            outputArea.setText("");
            functionCB.removeAllItems();
            vcCB.removeAllItems();
            vcArea.setText("");
        } 
        catch (IOException e) {
            error(e);
        }
    }

    private void save(File file) {
        try {
            FileWriter out = new FileWriter(file);
            out.write(codePane.getText());
            out.close();
        }
        catch (IOException e) {
            error(e);
        }
    }

    private void exit() {
        pi.stop();
        System.exit(1);
    }

    private Integer[] getLines(String lns) {
        if (lns == null) return new Integer[0];
        String[] split = lns.split(" ");
        Integer[] lines = new Integer[split.length];
        for (int i = 0; i < split.length; i++)
            lines[i] = Integer.valueOf(split[i].trim());
        return lines;
    }

    private void highlight(Integer[] lines) {
        Highlighter hl = codePane.getHighlighter();
        hl.removeAllHighlights();

        Integer[] ilines = new Integer[lines.length];
        int k = 0;
        for (int i = 0; i < lines.length; i++) {
            Integer line = lines[i];
            boolean old = false;
            for (int j = 0; j < k; j++) {
                if (ilines[j].equals(line)) {
                    old = true;
                    break;
                }
            }
            if (!old) {
                ilines[k] = line;
                k++;
            }
        }
        Arrays.sort(ilines, 0, k);

        String text = codePane.getText();
        String[] code = text.split("\n");
        int l = 0, sum = 0;
        for (int i = 0; i < k; i++) {
            int line = ilines[i].intValue() - 1;
            if (line >= code.length) return;
            for (; l < line; l++)
                sum += code[l].length() + 1;
	    int lft = 0;
	    for (;
		 lft < code[line].length() && 
		     (code[line].charAt(lft) == ' ' ||
		      code[line].charAt(lft) == '\t');
		 lft++);
	    lft += sum;
	    int rght = code[line].indexOf('@') + 1;
	    if (rght == 0) rght = code[line].length();
	    for (;
		 rght > 0 &&
		     (code[line].charAt(rght - 1) == ' ' ||
		      code[line].charAt(rght - 1) == '\t');
		 rght--);
	    rght += sum;
            try {
                if (rght >= text.length()) return;
                hl.addHighlight(lft, rght, hlp);
            }
            catch (BadLocationException e) {
                error(e);
            }
        }
    }

    private void vcCBLoad(String fname) {
        vcCB.removeAllItems();
        Collection vcns = pi.getVCNames(fname);
        boolean first = true;
        for (Iterator it = vcns.iterator(); it.hasNext(); ) {
            String next = (String) it.next();
            String label = vcLabel(next, pi.getStatus(next));
            vcCB.addItem(label);
            if (first) {
                first = false;
                vcCB.setSelectedItem(label);
            }
        }
    }

    private String vcLabel(String vcname, int status) {
        return 
	    (status == Pi.INVALID ? "[invalid]" : 
	     status == Pi.VALID ? "[valid]" :
	     "[pending]")
	    + " : " + vcname;
    }

    private String vcName(String vclabel) {
        if (vclabel == null) return null;
        String[] parts = vclabel.split(":");
	return parts[1].trim();
    }

    private String vcLines(String vclabel) {
        if (vclabel == null) return null;
        String[] parts = vclabel.split(":");
        parts = parts[1].trim().split("\\(");
	return parts[0].trim();
    }

    private void analyze(String text) {
        if (text == null) return;
        String ln = "";
        if (text.indexOf("<") == 0) {
            int near = text.indexOf("near ");
            if (near == -1) return;
            ln = text.substring(near + 5).trim();
        }
        else if (text.indexOf("Line <") == 0) {
            int rght = text.indexOf(">");
            ln = text.substring(6, rght).trim();
        }
        try {
            Integer line = Integer.valueOf(ln);
            Integer[] lines = new Integer[1];
            lines[0] = line;
            highlight(lines);
        }
        catch (NumberFormatException e) {}
    }

    public void piOutputLine(String line) {
        analyze(line);
        outputArea.append(line + "\n");
    }

    public void piNewVC(String fname, String vcname, boolean valid) {
        int cnt = functionCB.getItemCount();
        boolean contains = false;
        for (int i = 0; i < cnt; i++) {
            if (fname.equals((String) functionCB.getItemAt(i)))
                contains = true;
        }
        if (!contains) functionCB.addItem(fname);
        else if (functionCB.getSelectedItem().equals(fname))
            vcCBLoad(fname);
    }

    public void piCompiled(boolean correct) {
        compiling = false;
        if (correct ) {
	    if (email != null && !email.equals("")) sendEmail() ;
            else showValid();
        }
        else showInvalid();
    }

    public void showValid() {
        JOptionPane.showMessageDialog(mainPanel,
				      "All verification conditions are valid",
                                      "Click OK",
                                      JOptionPane.INFORMATION_MESSAGE);
    }

    public void showInvalid() {
        JOptionPane.showMessageDialog(mainPanel,
	    "Some verification conditions are not valid",
                                      "Click OK",
                                      JOptionPane.INFORMATION_MESSAGE);
    }
 
    public void sendEmail() {
       int answer =
	   JOptionPane.
	   showConfirmDialog(mainPanel, 
			     "Send email to TA?",
			     "piVC",
			     JOptionPane.YES_NO_OPTION);
       if (answer == JOptionPane.YES_OPTION) {
	   String msg = 
	       "Subject: VALID\n"
	       + codePane.getText();
	   String cmd = "sendmail " + email;
	   try {
	       Process sm = Runtime.getRuntime().exec(cmd);
	       BufferedWriter out = 
		   new BufferedWriter(
		        new OutputStreamWriter(sm.getOutputStream()));
	       out.write(msg);
	       out.close();
	   }
	   catch (IOException e) {
	       error(e);
	   }
       }
    }       

    public void piReturnValue(String rv) {
        debugArea.setText(rv);
    }

    public void piError(String err) {
        debugArea.setText(err);
    }

    public void piLines(Integer[] lines) {
        highlight(lines);
    }

    private static void runGUI() {
        PiGUI gui = new PiGUI(500);

        JFrame.setDefaultLookAndFeelDecorated(true);
        JFrame frame = new JFrame("PiVC");
        frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        frame.addWindowListener(gui.exitListener);

        frame.setJMenuBar(gui.menuBar);
        frame.setContentPane(gui.mainPanel);

        frame.pack();
        frame.setVisible(true);
    }

    public static void main(String[] args) {
        setFontSize();

	for (int i = 0; i < args.length; i++) {
	    if (args[i].equals("-email")) {
		i++;
		if (i == args.length) break;
		PiGUI.enableEmail(args[i]);
	    }
	    else if (args[i].equals("-pvc")) {
		i++;
		if (i == args.length) break;
		PiGUI.set_pvc(args[i]);
	    }
	    else if (args[i].equals("-cvcl")) {
		i++;
		if (i == args.length) break;
		PiGUI.set_cvcl("-cvcl " + args[i]);
	    }
	    else if (args[i].equals("-yices")) {
		i++;
		if (i == args.length) break;
		PiGUI.set_yices("-yices " + args[i]);
	    }
	    else if (args[i].equals("-timeout")) {
		i++;
		if (i == args.length) break;
		PiGUI.set_cvctimeout(args[i]);
	    }
	    else if (args[i].equals("-libdir")) {
		i++;
		if (i == args.length) break;
		PiGUI.setLibDir(args[i]);
	    }
	    else if (args[i].equals("-fontsz")) {
		i++;
		if (i == args.length) break;
		try {
		    setFontSize(Integer.parseInt(args[i]));
		}
		catch (NumberFormatException e) {
		    e.printStackTrace(System.out);
		}
	    }
	    else {
		System.out.println("Invalid argument: " + args[i]);
	    }
	}

        javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    runGUI();
                }
            });
    }
}
