(*
 * Copyright 2005, 2006, 2007 Aaron R. Bradley and Zohar Manna
 *
 * This file is part of PiVC.
 * 
 * PiVC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 * 
 * PiVC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

type id = int

type cfg =
  | Asgn of (Expr.t * Expr.t) list * id ref
  | Branch of Expr.t * id ref * id ref
  | Assert of Expr.t * id ref
  | Return of Expr.t option
  | Continue
  | Break

type blk = { 
    mutable vars : (Type.typ * string) list;
    cfg : id;
    dangling : id ref list;
  }

val cfg_of_id : id -> cfg

val linenum_of_id : id -> int

val get_cutpoints : unit -> id list


val assign_stmt : Expr.t -> Expr.t -> blk

val if_stmt : Expr.t -> blk -> blk -> blk

val for_stmt : Expr.t -> blk -> Expr.t -> blk -> blk -> blk

val while_stmt : Expr.t -> Expr.t -> blk -> blk

val continue_stmt : unit -> blk

val break_stmt : unit -> blk

val return_stmt : Expr.t option -> blk

val assert_stmt : Expr.t -> blk


val concat_stmts : blk -> blk -> blk

val empty_stmt : unit -> blk


val blk : (Type.typ * string) list -> blk -> blk

val empty_blk : unit -> blk


val type_check : string option -> blk -> unit


val typ : blk -> string -> Type.typ

val mk_function : 
    string list -> string -> Type.typ -> (Type.typ * string) list 
      -> Expr.t -> Expr.t -> blk -> blk


val string_of_blk : blk -> string


val function_source : 
    (string -> (Expr.t * Expr.t * Type.typ * (Type.typ * string) list)) -> unit


(* control variable *)

val autoasserts : bool ref
