(*
 * Copyright 2005, 2006, 2007 Aaron R. Bradley and Zohar Manna
 *
 * This file is part of PiVC.
 * 
 * PiVC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 * 
 * PiVC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

type t =
  | Bool of bool
  | Int of int
  | Float of float
  | Uninit
  | CArray of t array
  | CStruct of (string * t) list
  | Var of string
  | ArrayAlloc of Type.typ * t
  | ArrayAcc of t * t
  | Length of t
  | StructAcc of t * string
  | FnCall of string * t list
  | Collect of string * t list
  | Plus of t * t
  | Minus of t * t
  | Times of t * t
  | Div of t * t
  | IDiv of t * t
  | Mod of t * t
  | UMinus of t
  | LT of t * t
  | LE of t * t
  | NE of t * t
  | EQ of t * t
  | GE of t * t
  | GT of t * t
  | BExpr of t
  | And of t * t
  | Or of t * t
  | Not of t

module VSet : Set.S with type elt = string


val string_of_expr : t -> string


val bxor : t -> t -> t

val biff : t -> t -> t

val bimplies : t -> t -> t

val band : t -> t -> t

val bor : t -> t -> t

val bnot : t -> t

val bexpr : t -> t

val const_true : t

val const_false : t


val lt : t -> t -> t

val le : t -> t -> t

val ne : t -> t -> t

val eq : t -> t -> t

val ge : t -> t -> t

val gt : t -> t -> t


val fn_call : string -> t list -> t

val plus : t -> t -> t

val minus : t -> t -> t

val times : t -> t -> t

val div : t -> t -> t

val idiv : t -> t -> t

val modo : t -> t -> t

val uminus : t -> t

val var : string -> t


val array_alloc : Type.typ -> t -> t

val length : t -> t

val array_access : t -> t -> t

val struct_access : t -> string -> t


val const_int : int -> t

val const_float : float -> t

val const_array : t list -> t

val const_struct : (string * t) list -> t

val uninit : t


val collect : string -> t list -> t


val vars : t -> VSet.t

val sub : (string, t) Hashtbl.t -> t -> t


val typ_of : t -> Type.typ

val assert_typ : Type.typ -> t -> unit


val typ_source : (string -> Type.typ) -> unit

val function_source : 
    (string -> (Type.typ * Type.typ list)) -> unit
