(*
 * Copyright 2005, 2006, 2007 Aaron R. Bradley and Zohar Manna
 *
 * This file is part of PiVC.
 * 
 * PiVC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 * 
 * PiVC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

type fn = {
    fpre : Expr.t;
    fpost : Expr.t;
    ftyp : Type.typ;
    fnm : string;
    fparams : (Type.typ * string) list;
  }

type t = {
    typs : Type.TSet.t ref;
    typ_aliases : (string, Type.typ) Hashtbl.t;
    defs : (Type.typ * string * Expr.t) list ref;
    fns : (string, (fn * Cfg.blk option)) Hashtbl.t;
  }


val new_program : unit -> t


val declare_typ : t -> Type.typ -> unit

val declare_typ_alias : t -> Type.typ -> string -> unit

val declare_def : t -> Type.typ -> string -> Expr.t -> unit

val declare_fn : 
    t -> Expr.t -> Expr.t -> Type.typ -> string -> (Type.typ * string) list 
      -> unit

val declare_fn_impl : t -> string -> Cfg.blk -> unit


val type_check : t -> unit


val string_of_program : t -> string


val interp_source : (string -> bool) -> unit
