(*
 * Copyright 2005, 2006, 2007 Aaron R. Bradley and Zohar Manna
 *
 * This file is part of PiVC.
 * 
 * PiVC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 * 
 * PiVC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

type expr =
  | Bool of bool
  | Int of int
  | Float of float
  | Anything
  | CArray of expr array
  | CStruct of (string * expr) list
  | Var of string
  | Array of (expr * expr) list * expr
  | Length of expr
  | ArrayAcc of expr * expr
  | Struct of (string * expr) list * expr
  | StructAcc of expr * string
  | FnCall of string * expr list
  | Plus of expr * expr
  | Minus of expr * expr
  | Times of expr * expr
  | Div of expr * expr
  | IDiv of expr * expr
  | Mod of expr * expr
  | UMinus of expr
  | LT of expr * expr
  | LE of expr * expr
  | NE of expr * expr
  | EQ of expr * expr
  | GE of expr * expr
  | GT of expr * expr
  | BExpr of expr
  | And of expr * expr
  | Or of expr * expr
  | Not of expr
  | Forall of Type.typ * string * expr

(*
 * Input:
 *  o callback function
 *  o function
 * Finds all cutpoint paths of function, calling the callback
 * function on each path.
 *)
val paths : 
    (Interface.fn -> Interface.cfg list -> unit) -> Interface.fn -> unit

(*
 * Symbolic memory.
 *)
type mem

(*
 * Input:
 *  o prgm, fn
 *  o path, as cfg list (not containing Return statement)
 * Output:
 *  o mem, which can be queried with value
 *  o an expr, asserting what is true about temporary variables
 * All paths returns by paths (above) are safe.
 *)
val propagate : 
  Interface.prgm -> Interface.fn -> Interface.cfg list -> (expr * mem)

(*
 * Input:
 *  o mem
 *  o expr, which must be an lval (or fear an assert)
 * Output:
 *  o the symbolic value of expr
 *)
val value : mem -> expr -> expr
