(*
 * Copyright 2005, 2006, 2007 Aaron R. Bradley and Zohar Manna
 *
 * This file is part of PiVC.
 * 
 * PiVC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 * 
 * PiVC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

type expr =
  | Bool of bool
  | Int of int
  | Float of float
  | Anything
  | CArray of expr array
  | CStruct of (string * expr) list
  | Var of string
  | Array of (expr * expr) list * expr
  | Length of expr
  | ArrayAcc of expr * expr
  | Struct of (string * expr) list * expr
  | StructAcc of expr * string
  | FnCall of string * expr list
  | Plus of expr * expr
  | Minus of expr * expr
  | Times of expr * expr
  | Div of expr * expr
  | IDiv of expr * expr
  | Mod of expr * expr
  | UMinus of expr
  | LT of expr * expr
  | LE of expr * expr
  | NE of expr * expr
  | EQ of expr * expr
  | GE of expr * expr
  | GT of expr * expr
  | BExpr of expr
  | And of expr * expr
  | Or of expr * expr
  | Not of expr
  | Forall of Type.typ * string * expr

val string_of_expr : expr -> string
val step_string_of_expr : expr -> string

val type_of_expr : 
    (Type.typ * string) list -> Interface.prgm -> Interface.fn -> expr 
      -> Type.typ

(*
 * Input:
 *  o prgm
 *  o callback function, which accepts a description, prgm, fn, and VC
 *
 * Generates VCs, calling callback function with each.
 *)
val vcs : Interface.prgm 
  -> (string -> string -> Interface.prgm -> Interface.fn -> expr -> unit) 
    -> unit


(* options *)
val sanity : bool ref
