(*
 * Copyright 2005, 2006, 2007 Aaron R. Bradley and Zohar Manna
 *
 * This file is part of PiVC.
 * 
 * PiVC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your
 * option) any later version.
 * 
 * PiVC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

module E = Expr
module I = Interface
module T = Type

module L = List

let dot_of_prgm prgm =
  let dof fn =
    let label cfg = string_of_int (I.id cfg) in
    let dos cfg =
      match I.statement cfg with
      | I.Skip next ->
	  (label cfg) ^ "[label=\"<skip>\", shape=box];\n"
	  ^ (label cfg) ^ "->" ^ (label next) ^ ";\n"
      | I.Asgn (asgns, next) ->
	  (label cfg) ^ "[label=\"" 
	  ^ (String.concat "; " 
	       (L.map (fun (lv, rv) -> (E.string_of_expr lv) ^ " = " 
		 ^ (E.string_of_expr rv)) asgns))
	  ^ "\", shape=box];\n"
	  ^ (label cfg) ^ "->" ^ (label next) ^ ";\n"
      | I.Branch (grd, tb, fb) ->
	  (label cfg) ^ "[label=\""
	  ^ (E.string_of_expr grd)
	  ^ "\", shape=diamond];\n"
	  ^ (label cfg) ^ "->" ^ (label tb) ^ "[label=T];\n"
	  ^ (label cfg) ^ "->" ^ (label fb) ^ "[label=F];\n"
      | I.Assert (asrt, next) ->
	  (label cfg) ^ "[label=\""
	  ^ (E.string_of_expr asrt)
	  ^ "\", shape=hexagon];\n"
	  ^ (label cfg) ^ "->" ^ (label next) ^ ";\n"
      | I.Return rv ->
	  (label cfg) ^ "[label=\""
	  ^ (E.string_of_expr rv)
	  ^ "\", shape=ellipse];\n"
    in
    let seen = ref I.CfgSet.empty in
    let desc = ref "" in
    let rec doc cfg =
      if not (I.CfgSet.mem cfg !seen) then begin
	seen := I.CfgSet.add cfg !seen;
	desc := !desc ^ (dos cfg);
	I.CfgSet.iter doc (I.successors cfg)
      end
    in
    doc (I.body fn);
    let fndesc =
      let vlist vl =
	String.concat ", " 
	  (L.map 
	     (fun (typ, v) -> (T.string_of_typ typ) ^ " " ^ v) 
	     vl)
      in
      (T.string_of_typ (I.return_type fn)) ^ " " ^ (I.name fn) ^ "("
      ^ (vlist (I.parameters fn))
      ^ ")\\n  declare\\n  "
      ^ (vlist (I.locals fn))
    in
    "subgraph cluster_" ^ (I.name fn) ^ " {\n"
    ^ "label=\"" ^ fndesc ^ "\";\n"
    ^ !desc
    ^ "}\n"
  in
  let dot =
    "digraph Program {\n"
    ^ (String.concat "\n" 
	 (L.map dof (I.FnSet.elements (I.functions prgm))))
    ^ "\n}\n"
  in
  let tmp = open_out "vc.dot" in
  output_string tmp dot;
  close_out tmp;
  ignore (Unix.system "dot -Tps -oa.ps < vc.dot");
  Unix.unlink "vc.dot"
